(************************************************************************)
(*																		*)
(*					Group for the whole display							*)
(*																		*)
(************************************************************************)

unit	GuiMain;

interface

uses	GuiObj,GuiMenu,GuiMap,TClass;

type	TEventTypeCombo = object(TComboBox)	{ Special combo box for events }
	constructor Init(x,y,w,h:Integer);
	procedure	Put(var n:Integer);virtual;
	function	Get:Integer;virtual;
end;

type  	PEventTypeCombo = ^TEventTypeCombo;

type	TGuiDisplay = object(TGroup)		{ Main display object }
	TileChooser:PTileSelectEditor;
	TileMap:PMapEditor;
	TileCombo:PTileTypeCombo;
	ExitSel:array[0..3] of PNumeric;
	TileSet:PTileSet;
	EvObject,Weapon:PTileUse;
	EvAction:PEventTypeCombo;
	HitsReq,EvParam:PNumeric;
	ShootMonst,FastMonst:PCheckBox;
	constructor Init;
	destructor  Done;
	procedure	CopyIn(var r:TRoom);
	procedure	Extract(var r:TRoom);
	procedure	AddWithLabel(p:PGuiObject;s:String);
	procedure	GetPos(i:Integer;var x,y:Integer);
end;

type  	PGuiDisplay = ^TGuiDisplay;

function ParamEvent(EvID:Integer):Boolean;

implementation

uses Graph;

const	UnitHeight = 20;				{ Positioning of GUI Objects }
		UnitWidth = 180;
		UnitSpacing = UnitHeight+2;

const	DirName:array[0..3] of String = ('Up','Right','Down','Left');

(************************************)
(*	Create a Main Display Object	*)
(************************************)

constructor	TGuiDisplay.Init;
var	i,x,y:Integer;
begin
	TGroup.Init;
	New(TileSet,Init);
	TileChooser := New(PTileSelectEditor,Init(10,295,TileSet));
	GetPos(0,x,y);
	TileCombo := New(PTileTypeCombo,Init(x,y,UnitWidth,UnitHeight));
	Add(TileChooser);
	AddWithLabel(TileCombo,'Draw');
	TileMap := New(PMapEditor,Init(10,130,3,TileChooser,TileCombo));
	Add(TileMap);

	for i := 0 to 3 do
		begin
			GetPos(i+1,x,y);
            ExitSel[i] := New(PNumeric,Init(x,y,UnitWidth div 2,UnitHeight,0,255));
			AddWithLabel(ExitSel[i],DirName[i]);
			Add(New(PButton,Init(x+UnitWidth*6 div 10,y,
							UnitWidth*4 div 10,UnitHeight,i+1,'Go')));
		end;


	GetPos(5,x,y);
	EvObject := New(PTileUse,Init(x,y,TileChooser));
	AddWithLabel(EvObject,'Event Tool');

	GetPos(6,x,y);
	EvAction := New(PEventTypeCombo,Init(x,y,UnitWidth,UnitHeight));
	AddWithLabel(EvAction,'Action');

	GetPos(7,x,y);
	EvParam := New(PNumeric,Init(x+UnitWidth div 2,y,UnitWidth div 2,UnitHeight,-8,7));
	Add(EvParam);
    EvParam^.Put(0);
	Add(New(PButton,Init(x,y,UnitWidth*4 div 10,UnitHeight,10,'Message')));

	GetPos(8,x,y);
	Weapon := New(PTileUse,Init(x,y,TileChooser));
	AddWithLabel(Weapon,'Weapon Reqd');

	FastMonst := New(PCheckBox,Init(x+170,y,False));
	AddWithLabel(FastMonst,'Fast ?');

	GetPos(9,x,y);
	HitsReq := New(PNumeric,Init(x,y,UnitWidth div 2,UnitHeight,1,15));
	AddWithLabel(HitsReq,'Hits Reqd');

	ShootMonst := New(PCheckBox,Init(x+170,y,False));
	AddWithLabel(ShootMonst,'Shoots ?');

end;

(************************************)
(*  Delete a main display object	*)
(************************************)

destructor  TGuiDisplay.Done;
begin
	Dispose(TileSet,Done);
	TGroup.Done;
end;

(************************************************)
(*	Add an object, with an associated label 	*)
(************************************************)

procedure	TGuiDisplay.AddWithLabel(p:PGuiObject;s:String);
begin
	Add(p);
	Add(New(PText,Init(p^.OrgX-(TextWidth(s)+10),p^.OrgY,Concat(s,':'))));
end;

(*******************************)
(*	Get GUI Object position	   *)
(*******************************)

procedure	TGuiDisplay.GetPos(i:Integer;var x,y:Integer);
begin
	x := 430;y := 34+i*UnitSpacing;
end;

(****************************************)
(*	Copy Room data into GUI Objects		*)
(****************************************)

procedure	TGuiDisplay.CopyIn(var r:TRoom);
var i:Integer;
begin
    for i := 0 to 3 do ExitSel[i]^.Put(r.Exit[i]);
	TileMap^.Map := r.Map;
	EvObject^.Put(r.EventObject);
	i := r.EventAction;
	EvAction^.Put(i);
	if ParamEvent(r.EventAction) then
				EvParam^.Put(r.EventAction mod 16-8);
	EvObject^.Put(r.EventObject);
	Weapon^.Put(r.WeaponID);
	HitsReq^.Put(r.HitsRequired);
	FastMonst^.Put(r.FastMonster);
	ShootMonst^.Put(r.ShootMonster);
end;

(****************************************)
(*	Copy GUI Objects into Room Record	*)
(****************************************)

procedure	TGuiDisplay.Extract(var r:TRoom);
var i:Integer;
begin
    for i := 0 to 3 do r.Exit[i] := ExitSel[i]^.Get;
	r.Map := TileMap^.Map;
	r.EventAction := EvAction^.Get;
	if ParamEvent(r.EventAction) then
				r.EventAction := r.EventAction+(EvParam^.Get+8);
	r.EventObject := EvObject^.Get;
	r.WeaponID := Weapon^.Get;
	r.HitsRequired := HitsReq^.Get;
	r.FastMonster := FastMonst^.Get;
	r.ShootMonster := ShootMonst^.Get;
end;

(************************************)
(*	Special Combo Box for Events	*)
(************************************)

constructor TEventTypeCombo.Init(x,y,w,h:Integer);
var s:String;
begin
	s := 'Delete only:Roll up:Add 1 to Max Hearts:Cure All:Del if No Monsters:Message only:Change Hearts by ...';
	TComboBox.Init(x,y,w,h,1,s);
end;

(******************************************************************)
(*	Combo R/W. Is special because of coding of the Event Action	  *)
(*	which is not sequential and can have a parameter			  *)
(******************************************************************)

procedure	TEventTypeCombo.Put(var n:Integer);
begin
	case n of
		0..5:	n := n + 1;
		16..31:	n := 7;
	end;
	TComboBox.Put(n);
end;

function	TEventTypeCombo.Get:Integer;
var n:Integer;
begin
	n := TComboBox.Get;
	case n of
		1..6:	n := n-1;
		7:		n := 16;
	end;
	Get := n;
end;

(****************************************************)
(*	Is the given Event ID one with a parameter ?	*)
(****************************************************)

function ParamEvent(EvID:Integer):Boolean;
begin
	ParamEvent := (EvID >= 16) and (EvID < 32);
end;

begin
end.