(************************************************************************)
(*																		*)
(*								GUI Objects								*)
(*																		*)
(************************************************************************)

unit GuiObj;

interface

uses TClass;

type TGuiObject = object				{ Base class }
	OrgX,OrgY,Left,Top,Right,Bottom:Integer;
	constructor Init(x,y,Width,Height:Integer);
	destructor  Done;
	procedure	Click(x,y:Integer);virtual;
	procedure	Paint;virtual;
	function	InRect(x,y:Integer):Boolean;
end;

type TText = object(TGuiObject)			{ Text object class }
	OText:String;
	constructor	Init(x,y:Integer;s:String);
	procedure	Paint;virtual;
end;

type TButton = object(TGuiObject)
	Msg:String;
	ID:Integer;
	constructor Init(x,y,w,h,BID:Integer;s:String);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
end;

type TNumeric = object(TGuiObject)		{ Basic numeric control }
	Value,Min,Max:Integer;
	constructor	Init(x1,y1,w,h,Low,High:Integer);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure	Put(n:Integer);
	function	Get:Integer;
end;

type TCheckBox = object(TGuiObject)		{ Checkbox control }
	IsOn:Boolean;
	constructor	Init(x1,y1:Integer;Start:Boolean);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure	Put(On:Boolean);
	function	Get:Boolean;
end;

type TTileEdit = object(TGuiObject)
	Tile:TTileGraphic;
	Size:Integer;
	constructor Init(x,y,OSize:Integer);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
end;

type TTileSelector = object(TGuiObject)
	TSize:Integer;
	Selected:Integer;
	Tiles:PTileSet;
	constructor Init(x,y:Integer;TS:PTileSet);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure   Refresh(n:Integer);
end;

type PGuiObject = ^TGuiObject;
	 PText = ^TText;
	 PButton = ^TButton;
	 PNumeric = ^TNumeric;
	 PCheckBox = ^TCheckBox;
	 PTileEdit = ^TTileEdit;
	 PTileSelector = ^TTileSelector;

type TTileSelectEditor = object(TGuiObject)
	TSel:PTileSelector;
	TEd,TEdAlt: PTileEdit;
	CurrDisplayed:Integer;
	constructor Init(x,y:Integer;TS:PTileSet);
	destructor  Done;
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	function	Get:Integer;
end;

type PTileSelectEditor = ^TTileSelectEditor;

type TTileUse = object(TGuiObject)
	TileID:Integer;
	Tiles:PTileSelectEditor;
	constructor Init(x,y:Integer;TSE:PTileSelectEditor);
	procedure	Paint;virtual;
	procedure	Click(x,y:Integer);virtual;
	procedure	Put(n:Integer);
	function	Get:Integer;
end;

type PTileUse = ^TTileUse;

const MaxGroupSize = 32;

type TGroup = object
	Count:Integer;
	Obj:array[1..MaxGroupSize] of PGuiObject;
	constructor Init;
	destructor	Done;
	procedure	Paint;
	procedure   Add(p:PGuiObject);
	function	Click(x,y:Integer):Integer;
end;

type PGroup = ^TGroup;

var	 ActionButton:Integer;

implementation

uses Video,Graph;

(*********************************************************************)
(*	Base GUI Object which is a class from which others are derived	 *)
(*********************************************************************)

constructor TGuiObject.Init(x,y,Width,Height:Integer);
begin
	OrgX := x;OrgY := y;
	Left := x;Top := y - Height div 2;
	Right := x + Width-1;Bottom := Top+Height-1;
end;

destructor  TGuiObject.Done;
begin end;

procedure	TGuiObject.Click(x,y:Integer);
begin end;

procedure	TGuiObject.Paint;
begin end;

function	TGuiObject.InRect(x,y:Integer):Boolean;
begin
	InRect := (x >= Left) and (x <= Right) and (y >= Top) and (y <= Bottom);
end;

(****************************)
(*	Basic text item object	*)
(****************************)

constructor	TText.Init(x,y:Integer;s:String);
begin
	OText := s;
	TGuiObject.Init(x,y,TextWidth(s),TextHeight(s));
end;

procedure	TText.Paint;
begin
	SetColor(Black);OutTextXY(Left,Top,OText);
end;

(******************************)
(*	Numeric selector object	  *)
(******************************)

constructor	TNumeric.Init(x1,y1,w,h,Low,High:Integer);
begin
	Value := Low;Min := Low;Max := High;
	TGuiObject.Init(x1,y1,w,h);
end;

procedure	TNumeric.Put(n:Integer);
begin
	if n > Max then n := Max;
	if n < Min then n := Min;
	Value := n;Paint;
end;

function	TNumeric.Get:Integer;
begin
	Get := Value;
end;

procedure	TNumeric.Click(x,y:Integer);
var n,w,Border:Integer;
begin
	Border := (Right - Left) div 7;
	n := Get;w := Right-Left;
	if x < Border then n := n - 9;
	if x < Border*2 then Dec(n);
	if x > w-Border*2 then Inc(n);
	if x > w-Border then n := n + 9;
	if n <> Get then Put(n);
end;

procedure	TNumeric.Paint;
var Border:Integer;
	s:String;
begin
	Box3D(Left,Top,Right,Bottom,True);
	Border := (Right - Left) div 7;
	Box3D(Left+1,Top+1,Left+Border,Bottom-1,False);
	Box3D(Left+Border+1,Top+1,Left+Border*2,Bottom-1,False);
	Box3D(Right-Border,Top+1,Right-1,Bottom-1,False);
	Box3D(Right-Border*2,Top+1,Right-Border-1,Bottom-1,False);
	Str(Value,s);
	OutTextXY((Left+Right) div 2 - TextWidth(s) div 2+1,
			  (Top+Bottom) div 2 - TextHeight(s) div 2+1,s);
end;

(************************************)
(*	Simple 2-state checkbox object	*)
(************************************)

constructor	TCheckBox.Init(x1,y1:Integer;Start:Boolean);
begin
	IsOn := Start;
	TGuiObject.Init(x1,y1,10,10);
end;

procedure	TCheckBox.Paint;
begin
	Box3D(Left,Top,Right,Bottom,True);
	if IsOn then
		begin
			SetFillStyle(SolidFill,Black);
			Bar(Left+2,Top+2,Right-3,Bottom-3);
		end;
end;

procedure	TCheckBox.Click(x,y:Integer);
begin
	Put(not Get);
end;

procedure	TCheckBox.Put(On:Boolean);
begin
	if On <> IsOn then
		begin
			IsOn := On;Paint;
		end;
end;

function	TCheckBox.Get:Boolean;
begin
	Get := IsOn;
end;

(************************)
(*	Tile editing GUI	*)
(************************)

constructor TTileEdit.Init(x,y,OSize:Integer);
begin
	Tile.Init;
	Size := OSize;
	TGuiObject.Init(x,y,OSize*8+5,OSize*8+5);
end;

procedure   TTileEdit.Paint;
begin
	Box3D(Left,Top,Right,Bottom,False);
	Tile.Draw(Left+2,Top+2,Size,White);
end;

procedure	TTileEdit.Click(x,y:Integer);
begin
	x := (x - 2) div Size;y := (y - 2) div Size;
	if x < 0 then x := 0;
	if y < 0 then y := 0;
	if x > 7 then x := 7;
	if y > 7 then y := 7;
	Tile.SetPixel(x,y,not Tile.GetPixel(x,y));
	Paint;
end;

(********************)
(*	Tile selector	*)
(********************)

constructor TTileSelector.Init(x,y:Integer;TS:PTileSet);
begin
	Tiles := TS;TSize := 2;Selected := 0;
	TGuiObject.Init(x,y,16*10*TSize+2,4*10*TSize+2);
end;

procedure   TTileSelector.Refresh(n:Integer);
var x1,y1,c:Integer;
begin
	c := Green;if n = Selected then c := White;
	x1 := Left + n mod 16 * 10*TSize + 3;
	y1 := Top + n div 16 * 10*TSize + 3;
	Tiles^.Tile[n].Draw(x1,y1,TSize,c);
	c := LightGray;if n = Selected then c := Yellow;
	SetColor(c);Rectangle(x1-1,y1-1,x1+8*TSize,y1+8*TSize);
end;

procedure	TTileSelector.Click(x,y:Integer);
begin
	x := (x - 3) div (10*TSize);
	y := (y - 3) div (10*TSize);
	if x > 15 then x := 15;
	x := x + y * 16;
	if x >= TileSetSize then x := TileSetSize-1;
	if x <> Selected then
		begin
			y := Selected;Selected := -1;Refresh(y);
			Selected := x;Refresh(Selected);
		end;
end;

procedure	TTileSelector.Paint;
var i:Integer;
begin
	Box3D(Left,Top,Right,Bottom,False);
	for i := 0 to 63 do Refresh(i);
end;

(************************************)
(*	Tile Selector and Editor Combo	*)
(************************************)

constructor TTileSelectEditor.Init(x,y:Integer;TS:PTileSet);
begin
	TSel := nil;TEd := nil;
	TGuiObject.Init(x,y,520,95);
	New(TSel,Init(Left+6,(Top+Bottom) div 2,TS));
	New(TEd,Init(Left+334,(Top+Bottom) div 2-1,10));
	New(TEdAlt,Init(Left+426,(Top+Bottom) div 2-1,10));
	CurrDisplayed := TSel^.Selected;
	TEd^.Tile := TSel^.Tiles^.Tile[CurrDisplayed];
	TEdAlt^.Tile := TSel^.Tiles^.Tile[CurrDisplayed+64];
end;

destructor  TTileSelectEditor.Done;
begin
	Dispose(TSel,Done);
	Dispose(TEd,Done);
	Dispose(TEdAlt,Done);
end;

procedure	TTileSelectEditor.Paint;
begin
	Box3D(Left,Top,Right,Bottom,True);
	if TSel <> nil then TSel^.Paint;
	if TEd  <> nil then TEd^.Paint;
	if TEdAlt  <> nil then TEdAlt^.Paint;
end;

procedure	TTileSelectEditor.Click(x,y:Integer);
var x1:Integer;
begin
	x1 := x;x := x + Left;y := y + Top;
	if x1 < 330 then
		begin
	    	TSel^.Click(x-TSel^.Left,y-TSel^.Top);
			if CurrDisplayed <> TSel^.Selected then
				begin
					CurrDisplayed := TSel^.Selected;
					TEd^.Tile := TSel^.Tiles^.Tile[CurrDisplayed];
					TEdAlt^.Tile := TSel^.Tiles^.Tile[CurrDisplayed+64];
					TEd^.Paint;TEdAlt^.Paint;
				end;
		end;
	if (x1 > 330) and (x1 < 426) then
		begin
	    	TEd^.Click(x-TEd^.Left,y-TEd^.Top);
			TSel^.Tiles^.Tile[CurrDisplayed] := TEd^.Tile;
			TSel^.Refresh(CurrDisplayed);
		end;
	if x1 >= 426 then
		begin
			TEdAlt^.Click(x-TEdAlt^.Left,y-TEdAlt^.Top);
			TSel^.Tiles^.Tile[CurrDisplayed+64] := TEdAlt^.Tile;
		end;
end;

function	TTileSelectEditor.Get:Integer;
begin
	Get := CurrDisplayed;
end;

(********************************************)
(*	Tile picker, updated from a selector	*)
(********************************************)

constructor TTileUse.Init(x,y:Integer;TSE:PTileSelectEditor);
begin
	Tiles := TSE;TileID := 0;TGuiObject.Init(x,y,30,20);
end;

procedure	TTileUse.Paint;
begin
	Box3D(Left,Top,Right,Bottom,True);
	Box3D(Left+1,Top+1,Left+9,Bottom-1,False);
	Tiles^.TSel^.Tiles^.Tile[TileID].Draw(Left+11,Top+2,2,White);
end;

procedure	TTileUse.Click(x,y:Integer);
begin
	TileID := Tiles^.CurrDisplayed;
	Paint;
end;

procedure	TTileUse.Put(n:Integer);
begin
	TileID := n;Paint;
end;

function	TTileUse.Get:Integer;
begin
	Get := TileID;
end;

(****************************)
(*	Simple button object	*)
(****************************)

constructor TButton.Init(x,y,w,h,BID:Integer;s:String);
begin
	Msg := s;ID := BID;
	TGuiObject.Init(x,y,w,h);
end;

procedure	TButton.Paint;
begin
	Box3D(Left,Top,Right,Bottom,False);
	SetColor(Black);
	OutTextXY((Left+Right-TextWidth(Msg)) div 2,
			  (Top+Bottom-TextHeight(Msg)) div 2+1,
			  Msg);
end;

procedure	TButton.Click(x,y:Integer);
begin
	ActionButton := ID;
end;

(**************************)
(*	Grouped object type	  *)
(**************************)

constructor TGroup.Init;
begin
	Count := 0;
end;

destructor	TGroup.Done;
var i:Integer;
begin
	for i := 1 to Count do Dispose(Obj[i],Done);
	Count := 0;
end;

procedure	TGroup.Paint;
var i:Integer;
begin
	for i := 1 to Count do Obj[i]^.Paint;
end;

function	TGroup.Click(x,y:Integer):Integer;
var i:Integer;
begin
	ActionButton := 0;
	for i := 1 to Count do
		if Obj[i]^.InRect(x,y) then
				Obj[i]^.Click(x-Obj[i]^.Left,y-Obj[i]^.Top);
	Click := ActionButton;
end;

procedure   TGroup.Add(p:PGuiObject);
begin
	Inc(Count);
	Obj[Count] := p;
end;

begin
end.
