(********************************************************************)
(*																	*)
(*							Mouse input unit						*)
(*																	*)
(********************************************************************)

unit Video;

interface

procedure	VideoInitialise;
procedure	VideoDone;
procedure	VideoClearScreen;
procedure	VideoSetTitle(Title:String);
function	VideoGetClick(var x,y:Integer):Boolean;
procedure 	Box3D(x1,y1,x2,y2:Integer;Sunk:Boolean);

implementation

uses Graph,Dos;

var	xMouse,yMouse:Integer;				{ and display initialisation }
	Drawn,Button:Boolean;
	PtrMem:Pointer;

(************************)
(*	Initialise video	*)
(************************)

procedure VideoInitialise;
var Driver,Mode:Integer;
	Reg:Registers;
begin
	Driver := EGA;Mode := EGAHi;		{ Switch to EGA High mode }
	InitGraph(Driver,Mode,'.');
	SetGraphMode(Mode);
	Reg.ax := 0;						{ Initialise mouse driver }
	Intr($33,Reg);
	XMouse := -99;						{ Cursor off screen, not drawn }
	Drawn := False;Button := False;
	GetMem(PtrMem,ImageSize(0,0,30,30));{ Mouse pointer screen memory }
	VideoClearScreen;					{ Clear the display }
end;

(**************************)
(*	Finished with video	  *)
(**************************)

procedure  VideoDone;
begin
	FreeMem(PtrMem,ImageSize(0,0,30,30));
	CloseGraph;
end;

(****************************)
(*	Draw the mouse cursor	*)
(****************************)

procedure	VideoDrawMouse(x,y:Integer);
var MouseCur:array[1..4] of PointType;
begin
	MouseCur[1].x := x;MouseCur[1].y := y;
	MouseCur[2].x := x+20;MouseCur[2].y := y+10;
	MouseCur[3].x := x+10;MouseCur[3].y := y+20;
	MouseCur[4] := MouseCur[1];
	SetFillStyle(SolidFill,White);FillPoly(4,MouseCur);
	SetColor(Black);DrawPoly(4,MouseCur);
end;

(**************************************************)
(*	Move the mouse cursor to the mouse position	  *)
(**************************************************)

procedure	VideoMoveMouse;
var r:Registers;
begin
	r.ax := 3;							{ Read position and buttons }
	Intr($33,r);
	Button := ((r.bx and $03) <> 0);	{ Set buttons flag }
	if r.cx > GetMaxX-22 then r.cx := GetMaxX-22;
	if r.dx > GetMaxY-22 then r.dx := GetMaxY-22;
	if (r.cx <> XMouse) or (r.dx <> YMouse) or (not Drawn) then
		begin
			if Drawn then				{ Erase old cursor }
				PutImage(XMouse,YMouse,PtrMem^,NormalPut);
			XMouse := r.cx;YMouse := r.dx;
			GetImage(XMouse,YMouse,		{ Save new video memory }
						XMouse+20,YMouse+20,PtrMem^);
			VideoDrawMouse(XMouse,YMouse);	{ Draw new cursor }
			Drawn := True;
		end;
end;

(**********************************)
(*	Move cursor and get a click	  *)
(**********************************)

function	VideoGetClick(var x,y:Integer):Boolean;
begin
	repeat VideoMoveMouse until Not Button;	{ Wait for button release }
	repeat VideoMoveMouse until Button;		{ Wait for button press }
	if Drawn then						{ Hide the cursor }
			PutImage(XMouse,YMouse,PtrMem^,NormalPut);
	Drawn := False;
    x := XMouse;y := YMouse;			{ Get coordinates }
	VideoGetClick := (x < 15) and (y < 15);	{ Return true if clicked box }
end;

(************************)
(*	Clear the screen	*)
(************************)

procedure	VideoClearScreen;
begin
	SetFillStyle(SolidFill,LightGray);Bar(0,0,GetMaxX,GetMaxY);
	SetColor(White);Rectangle(0,0,GetMaxX-1,GetMaxY-1);
	VideoSetTitle('');
end;

(************************)
(*	Set the title bar	*)
(************************)

procedure	VideoSetTitle(Title:String);
var i,y:Integer;
begin
	SetFillStyle(SolidFill,White);Bar(0,0,GetMaxX,12);
	SetColor(Black);
	OutTextXY(18,3,Title);Rectangle(1,1,11,11);
	SetFillStyle(SolidFill,Black);Bar(4,4,8,8);
	for i := 1 to 5 do
		begin
			y := i * 2;
	    	Line(18+TextWidth(Title)+4,y,GetMaxX-2,y);
		end;
end;

procedure Box3D(x1,y1,x2,y2:Integer;Sunk:Boolean);
begin
	SetFillStyle(SolidFill,LightGray);Bar(x1,y1,x2,y2);
	if Sunk then SetColor(Black) else SetColor(White);
	Rectangle(x1,y1,x2,y2);
	if Sunk then SetColor(White) else SetColor(Black);
	Line(x2,y2,x2,y1);Line(x2,y2,x1,y2);
end;

begin
end.
