(************************************************************************)
(*																		*)
(*							    Builder Program							*)
(*																		*)
(************************************************************************)

program Builder;

uses TClass,Compress;

var	OutFile:File of Byte;				{ Destination file }
	Stored:Array[1..255] of Boolean;	{ Flags for rooms in data file }
	ToCopy:Array[1..255] of Boolean;	{ Flags for rooms to copy }
	CopyCount:Integer;					{ Number to copy }
	Bitmap:Array[0..4149] of Byte;		{ Bitmap data }

	i:Byte;

	ComBuffer:ByteArray;				{ Compression stuff }
	ComBufferSize:Byte;

(****************************)
(*  Copy the Title Bitmap	*)
(****************************)

procedure	CopyBitmap;
var f:File of Byte;
	i,x,y,c:Integer;
	b:Byte;

begin
	Writeln('    Copying Title Bitmap.');
	Assign(f,'Title.Bmp');
	Reset(f);
	for i := 0 to 4149 do Read(f,Bitmap[i]);
	for y := 0 to 31 do
		for x := 0 to 11 do
			begin
				b := 0;
				for c := 0 to 7 do
					if Bitmap[1078+(31-y)*96+x*8+c] = 0 then
						b := b or (128 shr c);
				Write(Outfile,b);
			end;
	Close(f);
end;


(****************************************)
(*	Look for a room to copy then do it	*)
(****************************************)

procedure CopyRoom;
var r,i,x,y,n:Integer;
	Room:TRoom;
	s:Byte;
begin
	n := 1;                       		{ Find the room to copy }
	while not ToCopy[n] do Inc(n);
    WriteLn('    Copying Room number ',n);
	Room.Init;   						{ Get the room }
	Room.Load(n);

	if Room.Exit[0] + Room.Exit[1] + Room.Exit[2] + Room.Exit[3] = 0 then
				WriteLn('WARNING:This room either does not exist or has no exits.');

	ComBufferSize := 255;				{ Initialise buffer size }
	for i := 0 to 255 do ComBuffer[i] := 0;

	ComBuffer[0] := Room.RoomID;		{ Copy data into compression buffer }
	ComBuffer[1] := 0;
	ComBuffer[2] := Room.WeaponID;
	ComBuffer[3] := Room.HitsRequired;
	if Room.FastMonster then ComBuffer[3] := ComBuffer[3] or $10;
	if Room.ShootMonster then ComBuffer[3] := ComBuffer[3] or $20;
	ComBuffer[4] := Room.EventObject;
	ComBuffer[5] := Room.EventAction;
	for i := 0 to 3 do
			ComBuffer[6+i] := Room.Exit[i];
	for	y := 0 to 7 do
		for x := 0 to 11 do
			ComBuffer[10+x+y*12] := Room.Map[x,y];

	for x := 1 to Length(Room.Message) do
			ComBuffer[105+x] := Ord(Room.Message[x]);
	ComBuffer[106+Length(Room.Message)] := 0;

	CompressData(ComBufferSize,ComBuffer);	{ Compress the data }

	s := ComBufferSize+1;				{ +1 to include offset }
	Write(OutFile,s);					{ Write 'offset to next' pointer }
	for i := 0 to ComBufferSize-1 do	{ Write data }
					Write(OutFile,ComBuffer[i]);

	for i := 0 to 3 do					{ Check adjacent rooms }
		begin
        	r := Room.Exit[i];
			if r <> 0 then
	           	if (Not Stored[r]) and (Not ToCopy[r]) then
					begin
						ToCopy[r] := True;
						Inc(CopyCount);
					end;
		end;

	ToCopy[n] := False;					{ Doesn't need doing }
	Stored[n] := True;					{ Has been done }
	Dec(CopyCount);						{ One fewer to copy }
end;

(****************************************)
(*	Copy a binary file to the output	*)
(****************************************)

procedure AppendFile(FName:String);
var f:File of Byte;
	Data:Byte;

begin
	WriteLn('    Copying ',FName,'.');
	Assign(f,FName);
	Reset(f);
	while not eof(f) do
		begin
			Read(f,Data);
			Write(OutFile,Data);
		end;
	Close(f);
end;

begin
	WriteLn('Builder. Builds a Data file from the rooms and tiles.');
	Assign(OutFile,'GAMEDATA.BIN');		{ Open the output file }
	Rewrite(OutFile);
	CopyBitmap;							{ Copy tile bitmap }
	AppendFile('TILE.BIN');				{ Copy the tile data }
	for i := 1 to 255 do				
		begin
        	Stored[i] := False;			{ No rooms yet loaded }
			ToCopy[i] := False;			{ No room to copy }
		end;

	ToCopy[1] := True;					{ Copy the first room }
	CopyCount := 1;						{ One room pending }

	while CopyCount > 0 do CopyRoom;	{ Output the room data }
	i := 0;Write(OutFile,i);			{ A zero size indicates end of the }
										{ Room linked list }

	WriteLn('Compiled file is ',FilePos(OutFile),' bytes in length.');
	Close(OutFile);
end.
