(********************************************************************)
(*																	*)
(*							Layout Viewer Program					*)
(*																	*)
(********************************************************************)

program VLayout;

uses TClass,Graph,Crt;

type RoomDataRec = Record
	IsLoaded:Boolean;					{ True if read }
	XPos,YPos:Integer;					{ Position offset from 0,0 (Room1) }
	Exits:array[0..3] of Integer;
end;

var RoomInfo:array[1..255] of RoomDataRec;
	Room:TRoom;
	XMin,YMin,XMax,YMax:Integer;

(*************************************************************)
(*	Recursive function, scan room array from given position	 *)
(*************************************************************)

procedure ScanRooms(RoomID,x,y:Integer);
var Exits:array[0..3] of Byte;
	i:Integer;
begin
	if not RoomInfo[RoomID].IsLoaded then
		begin
			if x < xMin then xMin := x;
			if x > xMax then xMax := x;
			if y < yMin then yMin := y;
			if y > yMax then yMax := y;
			RoomInfo[RoomID].IsLoaded := True;
			RoomInfo[RoomID].XPos := x;
			RoomInfo[RoomID].YPos := y;
			{ WriteLn('Scanning ',RoomID,' at ',x,',',y); }
			Room.Init;
			Room.Load(RoomID);
			for i := 0 to 3 do Exits[i] := Room.Exit[i];
			for i := 0 to 3 do RoomInfo[RoomID].Exits[i] := Exits[i];
			if Exits[0] <> 0 then ScanRooms(Exits[0],x,y+1);
			if Exits[1] <> 0 then ScanRooms(Exits[1],x+1,y);
			if Exits[2] <> 0 then ScanRooms(Exits[2],x,y-1);
			if Exits[3] <> 0 then ScanRooms(Exits[3],x-1,y);
		end;
end;

(************************************)
(*	Load in the room information	*)
(************************************)

procedure	LoadRoomInfo;
var i:Integer;
begin
	for i := 1 to 255 do RoomInfo[i].IsLoaded := False;
	XMin := 1;YMin := 1;XMax := -1;YMax := -1;
	ScanRooms(1,0,0);Inc(XMax);Inc(YMax);
	{WritelN(xmin,xmax,ymin,ymax);}
end;

(**********************************************)
(*	Convert logical to physical coordinates	  *)
(**********************************************)

procedure	Convert(xl,yl:Real;var x,y:Integer);
begin
	xl := (xl - xMin) / (xMax - xMin);
	yl := (yl - yMin) / (yMax - yMin);
	x := Trunc(xl * GetMaxX);
	y := GetMaxY-Trunc(yl * GetMaxY);
end;

(**********************************)
(*	Draw line between two rooms	  *)
(**********************************)

procedure	DrawLink(R1:Integer;xo1,yo1:Real;R2:Integer;xo2,yo2:Real);
var x1,y1,x2,y2:Integer;
begin
	Convert(RoomInfo[r1].XPos+xo1,RoomInfo[r1].YPos+yo1,x1,y1);
	Convert(RoomInfo[r2].XPos+xo2,RoomInfo[r2].YPos+yo2,x2,y2);
	SetColor(Red);Line(x1,y1,x2,y2);
	Circle(x2,y2,4);
end;

(******************)
(*	Draw Room i   *)
(******************)

procedure	DrawRoom(i:Integer);
var x,y,x1,y1,x2,y2:Integer;
	s:String;
begin
	x := RoomInfo[i].xPos;
	y := RoomInfo[i].yPos;
	Convert(x+0.2,y+0.2,x1,y1);
	Convert(x+0.8,y+0.8,x2,y2);
	SetColor(Green);Rectangle(x1,y1,x2,y2);Rectangle(x1+1,y1+1,x2-1,y2-1);
	SetColor(Yellow);Str(i,s);OutTextXY(x1+3,y2+3,s);

	if RoomInfo[i].Exits[0] <> 0 then
		DrawLink(i,0.5,0.8,RoomInfo[i].Exits[0],0.5,0.2);
	if RoomInfo[i].Exits[1] <> 0 then
		DrawLink(i,0.8,0.5,RoomInfo[i].Exits[1],0.2,0.5);
	if RoomInfo[i].Exits[2] <> 0 then
		DrawLink(i,0.5,0.2,RoomInfo[i].Exits[2],0.5,0.8);
	if RoomInfo[i].Exits[3] <> 0 then
		DrawLink(i,0.2,0.5,RoomInfo[i].Exits[3],0.8,0.5);
end;

var	i,Driver,Mode:Integer;

begin
	Driver := EGA;Mode := EGAHi;
	LoadRoomInfo;
	InitGraph(Driver,Mode,'.');
	SetGraphMode(Mode);
	SetColor(White);Rectangle(0,0,GetMaxX-1,GetMaxY-1);
	for i := 1 to 255 do
		if RoomInfo[i].IsLoaded then DrawRoom(i);
	i := Ord(ReadKey);
	CloseGraph;
end.